#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "const.h"
#include "file.h"
#include "prep-tabs2.h"

extern char user_dir[255];

static unsigned short yuvtab16[64 * 64 * 64][36];

//----------------------------------------------
//------------------------------------------
static unsigned short
yuv_to_rgb16 (int y, int u, int v, int grad_winkel, int bits)
{
  int r, g, b, iu, iv;

  double phase, fu, fv;
  unsigned short rgb;
  iu = u - 128;
  iv = v - 128;

  phase = grad_winkel * 2 * 3.141592654 / 360.0;
  fu = (double) iu *cos (phase) - (double) iv *sin (phase);
  fv = (double) iu *sin (phase) + (double) iv *cos (phase);

  fu += 128;
  fv += 128;
  b = (int) (116 * (y - 16) + 202 * (fu - 128)) / 100;
  g = (int) (116 * (y - 16) - 81 * (fv - 128) - 39 * (fu - 128)) / 100;
  r = (int) (116 * (y - 16) + 160 * (fv - 128)) / 100;
  if (g > 0xff)
    {
      g = 0xff;
    }
  if (b > 0xff)
    {
      b = 0xff;
    }
  if (r > 0xff)
    {
      r = 0xff;
    }
  if (g < 0)
    {
      g = 0;
    }
  if (b < 0)
    {
      b = 0;
    }
  if (r < 0)
    {
      r = 0;
    }
  if (bits == 15)
    {
      rgb = ((b >> 3) | ((g >> 3) << 5) | ((r >> 3) << 10));
    }
  else
    {
      rgb = ((b >> 3) | ((g >> 2) << 5) | ((r >> 3) << 11));
    }
  return (rgb);
}

//------------------------------------------
//----------------------------------------------
int
prep_tabs2 (void)
{
  FILE *f;
  int i, bits;
  int yi, ui, vi, y, u, v;
  char s1[200];

  bits = 16;

  sprintf (s1, "%s/yuvtab10_%d.tab", user_dir, bits);    // 10 ist die "winkel-dichte"

  if (!file_exist (s1))
    {
      if (!(f = fopen (s1, "w")))
        {
          printf ("cant write yuvtab...\n");
          exit (1);
        }

      for (yi = 0; yi < 64; yi++)
        for (ui = 0; ui < 64; ui++)
          for (vi = 0; vi < 64; vi++)
            {
              y = yi << 2;
              u = ui << 2;
              v = vi << 2;
              for (i = 0; i < 36; i++)
                yuvtab16[(yi << 12) | (ui << 6) | vi][i] =
                  yuv_to_rgb16 (y, u, v, 10 * i, bits);
            }

      fwrite (yuvtab16, 2, 64 * 64 * 64 * 36, f);

      fclose (f);
    }

  return (0);
}
